/* THIS IS A MACHINE-GENERATED FILE. DO NOT EDIT ! */

#if HAVE_CONFIG_H
#include <config.h>
#endif

const char *cause_text[] = {
    "unknown cause 0",
    "unallocated (unassigned) number",
    "no route to specified transit network",
    "no route to destination",
    "unknown cause 4",
    "unknown cause 5",
    "unknown cause 6",
    "unknown cause 7",
    "unknown cause 8",
    "unknown cause 9",
    "VPCI/VCI unacceptable",
    "unknown cause 11",
    "unknown cause 12",
    "unknown cause 13",
    "unknown cause 14",
    "unknown cause 15",
    "normal call clearing",
    "user busy",
    "no user responding",
    "unknown cause 19",
    "unknown cause 20",
    "call rejected",
    "number changed",
    "user rejects all calls with calling line identification restriction (CLIR)",
    "unknown cause 24",
    "unknown cause 25",
    "unknown cause 26",
    "destination out of order",
    "invalid number format (address incomplete)",
    "unknown cause 29",
    "response to STATUS ENQUIRY",
    "normal, unspecified",
    "unknown cause 32",
    "unknown cause 33",
    "unknown cause 34",
    "requested VPCI/VCI unavailable",
    "VPCI/VCI assignment failure",
    "user cell rate not available (>3.0)",
    "network out of order - unused",
    "unknown cause 39",
    "unknown cause 40",
    "temporary failure",
    "unknown cause 42",
    "access information discarded",
    "unknown cause 44",
    "no VPCI/VCI available",
    "unknown cause 46",
    "resource unavailable, unspecified",
    "unknown cause 48",
    "Quality of Service unavailable",
    "unknown cause 50",
    "user cell rate not available (3.0)",
    "unknown cause 52",
    "unknown cause 53",
    "unknown cause 54",
    "unknown cause 55",
    "unknown cause 56",
    "bearer capability not authorized",
    "bearer capability not presently available",
    "unknown cause 59",
    "unknown cause 60",
    "unknown cause 61",
    "unknown cause 62",
    "service or option not available, unspecified",
    "unknown cause 64",
    "bearer capability not implemented",
    "unknown cause 66",
    "unknown cause 67",
    "unknown cause 68",
    "unknown cause 69",
    "unknown cause 70",
    "unknown cause 71",
    "unknown cause 72",
    "unsupported combination of traffic parameters",
    "unknown cause 74",
    "unknown cause 75",
    "unknown cause 76",
    "unknown cause 77",
    "AAL param. cannot be supported (>3.0)",
    "unknown cause 79",
    "unknown cause 80",
    "invalid call reference value",
    "identified channel does not exist",
    "unknown cause 83",
    "unknown cause 84",
    "unknown cause 85",
    "unknown cause 86",
    "unknown cause 87",
    "incompatible destination",
    "invalid endpoint reference",
    "unknown cause 90",
    "invalid transit network selection",
    "too many pending add party requests",
    "AAL param. cannot be supported (3.0)",
    "unknown cause 94",
    "unknown cause 95",
    "mandatory information element is missing",
    "message type non-existent or not implemented",
    "unknown cause 98",
    "information element non-existent or not implemented",
    "invalid information element contents",
    "message not compatible with call state",
    "recovery on timer expiry",
    "unknown cause 103",
    "incorrect message length",
    "unknown cause 105",
    "unknown cause 106",
    "unknown cause 107",
    "unknown cause 108",
    "unknown cause 109",
    "unknown cause 110",
    "protocol error, unspecified",
    "unknown cause 112",
    "unknown cause 113",
    "unknown cause 114",
    "unknown cause 115",
    "unknown cause 116",
    "unknown cause 117",
    "unknown cause 118",
    "unknown cause 119",
    "unknown cause 120",
    "unknown cause 121",
    "unknown cause 122",
    "unknown cause 123",
    "unknown cause 124",
    "unknown cause 125",
    "unknown cause 126",
    "unknown cause 127"
};
